import { QuirrelClient, EnqueueJobOptions, Job, DefaultJobOptions, QuirrelJobHandler, QuirrelOptions } from "./client";
type NextApiHandler = (req: Request) => Response | Promise<Response>;
export { Job, EnqueueJobOptions, DefaultJobOptions, QuirrelJobHandler, };
export type Queue<Payload> = Omit<QuirrelClient<Payload>, "respondTo" | "makeRequest">;
/**
 * @example // usage in route handler file: api/queues/sample
 * export const sampleQueue = Queue(
 *   "/api/queues/sample",
 *   (payload) => {
 *      ...logic goes here
 *   }
 * );
 *
 * export const POST = sampleQueue;
 */
export declare function Queue<Payload>(route: string, handler: QuirrelJobHandler<Payload>, options?: QuirrelOptions): Queue<Payload> & NextApiHandler;
/**
 * @example // usage in route handler file: api/cronjobs/sample
 * const sampleCron = CronJob(
 *   "/api/cronjobs/sample",
 *   "0 0 * * 1"
 *   (payload) => {
 *      ...logic goes here
 *   }
 * );
 *
 * export const POST = sampleCron;
 */
export declare function CronJob(route: string, cronSchedule: NonNullable<NonNullable<EnqueueJobOptions["repeat"]>["cron"]>, handler: () => Promise<void>, options?: QuirrelOptions): unknown;
