"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CronJob = exports.Queue = void 0;
const client_1 = require("./client");
const config_1 = require("./client/config");
(0, config_1.registerDevelopmentDefaults)({
    applicationPort: 3000,
});
/**
 * @example // usage in route handler file: api/queues/sample
 * export const sampleQueue = Queue(
 *   "/api/queues/sample",
 *   (payload) => {
 *      ...logic goes here
 *   }
 * );
 *
 * export const POST = sampleQueue;
 */
function Queue(route, handler, options) {
    const quirrel = new client_1.QuirrelClient({
        options,
        handler,
        route,
    });
    async function nextApiHandler(req) {
        const { body, headers, status } = await quirrel.respondTo(await req.text(), Object.fromEntries([...req.headers.entries()]));
        return new Response(body, {
            headers,
            status,
        });
    }
    nextApiHandler.enqueue = (payload, options) => quirrel.enqueue(payload, options);
    nextApiHandler.enqueueMany = (jobs) => quirrel.enqueueMany(jobs);
    nextApiHandler.delete = (jobId) => quirrel.delete(jobId);
    nextApiHandler.invoke = (jobId) => quirrel.invoke(jobId);
    nextApiHandler.get = () => quirrel.get();
    nextApiHandler.getById = (jobId) => quirrel.getById(jobId);
    return nextApiHandler;
}
exports.Queue = Queue;
/**
 * @example // usage in route handler file: api/cronjobs/sample
 * const sampleCron = CronJob(
 *   "/api/cronjobs/sample",
 *   "0 0 * * 1"
 *   (payload) => {
 *      ...logic goes here
 *   }
 * );
 *
 * export const POST = sampleCron;
 */
function CronJob(route, cronSchedule, handler, options) {
    return Queue(route, handler, options);
}
exports.CronJob = CronJob;
//# sourceMappingURL=next-app.js.map